package com.sun.media.sound;

import javax.sound.midi.Patch;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;

public class SimpleInstrument extends ModelInstrument
{
    protected int preset;
    protected int bank;
    protected boolean percussion;
    protected String name;
    protected List<SimpleInstrument.SimpleInstrumentPart> parts;
    
    public SimpleInstrument() {
        super(null, null, null, null);
        this.preset = 0;
        this.bank = 0;
        this.percussion = false;
        this.name = "";
        this.parts = new ArrayList();
    }
    
    public void clear() {
        this.parts.clear();
    }
    
    public void add(ModelPerformer[] performers, int keyFrom, int keyTo, int velFrom, int velTo, int exclusiveClass) {
        SimpleInstrument.SimpleInstrumentPart simpleInstrumentPart = new SimpleInstrument.SimpleInstrumentPart();
        simpleInstrumentPart.performers = performers;
        simpleInstrumentPart.keyFrom = keyFrom;
        simpleInstrumentPart.keyTo = keyTo;
        simpleInstrumentPart.velFrom = velFrom;
        simpleInstrumentPart.velTo = velTo;
        simpleInstrumentPart.exclusiveClass = exclusiveClass;
        this.parts.add(simpleInstrumentPart);
    }
    
    public void add(ModelPerformer[] array, int n, int n2, int n3, int n4) {
        this.add(array, n, n2, n3, n4, -1);
    }
    
    public void add(ModelPerformer[] array, int n, int n2) {
        this.add(array, n, n2, 0, 127, -1);
    }
    
    public void add(ModelPerformer[] array) {
        this.add(array, 0, 127, 0, 127, -1);
    }
    
    public void add(ModelPerformer modelPerformer, int n, int n2, int n3, int n4, int n5) {
        this.add(new ModelPerformer[] { modelPerformer }, n, n2, n3, n4, n5);
    }
    
    public void add(ModelPerformer modelPerformer, int n, int n2, int n3, int n4) {
        this.add(new ModelPerformer[] { modelPerformer }, n, n2, n3, n4);
    }
    
    public void add(ModelPerformer modelPerformer, int n, int n2) {
        this.add(new ModelPerformer[] { modelPerformer }, n, n2);
    }
    
    public void add(ModelPerformer modelPerformer) {
        this.add(new ModelPerformer[] { modelPerformer });
    }
    
    public void add(ModelInstrument modelInstrument, int n, int n2, int n3, int n4, int n5) {
        this.add(modelInstrument.getPerformers(), n, n2, n3, n4, n5);
    }
    
    public void add(ModelInstrument modelInstrument, int n, int n2, int n3, int n4) {
        this.add(modelInstrument.getPerformers(), n, n2, n3, n4);
    }
    
    public void add(ModelInstrument modelInstrument, int n, int n2) {
        this.add(modelInstrument.getPerformers(), n, n2);
    }
    
    public void add(ModelInstrument modelInstrument) {
        this.add(modelInstrument.getPerformers());
    }
    
    @Override
    public ModelPerformer[] getPerformers() {
        int n = 0;
        for (SimpleInstrument.SimpleInstrumentPart simpleInstrumentPart : this.parts) {
            if (simpleInstrumentPart.performers != null) {
                n += simpleInstrumentPart.performers.length;
            }
            else {
                continue;
            }
        }
        ModelPerformer[] array = new ModelPerformer[n];
        int n2 = 0;
        for (SimpleInstrument.SimpleInstrumentPart simpleInstrumentPart2 : this.parts) {
            if (simpleInstrumentPart2.performers != null) {
                for (ModelPerformer modelPerformer : simpleInstrumentPart2.performers) {
                    ModelPerformer o = new ModelPerformer();
                    o.setName(this.getName());
                    (array[n2++] = o).setDefaultConnectionsEnabled(modelPerformer.isDefaultConnectionsEnabled());
                    o.setKeyFrom(modelPerformer.getKeyFrom());
                    o.setKeyTo(modelPerformer.getKeyTo());
                    o.setVelFrom(modelPerformer.getVelFrom());
                    o.setVelTo(modelPerformer.getVelTo());
                    o.setExclusiveClass(modelPerformer.getExclusiveClass());
                    o.setSelfNonExclusive(modelPerformer.isSelfNonExclusive());
                    o.setReleaseTriggered(modelPerformer.isReleaseTriggered());
                    if (simpleInstrumentPart2.exclusiveClass != -1) {
                        o.setExclusiveClass(simpleInstrumentPart2.exclusiveClass);
                    }
                    if (simpleInstrumentPart2.keyFrom > o.getKeyFrom()) {
                        o.setKeyFrom(simpleInstrumentPart2.keyFrom);
                    }
                    if (simpleInstrumentPart2.keyTo < o.getKeyTo()) {
                        o.setKeyTo(simpleInstrumentPart2.keyTo);
                    }
                    if (simpleInstrumentPart2.velFrom > o.getVelFrom()) {
                        o.setVelFrom(simpleInstrumentPart2.velFrom);
                    }
                    if (simpleInstrumentPart2.velTo < o.getVelTo()) {
                        o.setVelTo(simpleInstrumentPart2.velTo);
                    }
                    o.getOscillators().addAll(modelPerformer.getOscillators());
                    o.getConnectionBlocks().addAll(modelPerformer.getConnectionBlocks());
                }
            }
            else {
                continue;
            }
        }
        return array;
    }
    
    @Override
    public Object getData() {
        return null;
    }
    
    @Override
    public String getName() {
        return this.name;
    }
    
    public void setName(String name) {
        this.name = name;
    }
    
    @Override
    public ModelPatch getPatch() {
        return new ModelPatch(this.bank, this.preset, this.percussion);
    }
    
    public void setPatch(Patch patch) {
        if (patch instanceof ModelPatch && ((ModelPatch)patch).isPercussion()) {
            this.percussion = true;
            this.bank = patch.getBank();
            this.preset = patch.getProgram();
        }
        else {
            this.percussion = false;
            this.bank = patch.getBank();
            this.preset = patch.getProgram();
        }
    }
    
    private static class SimpleInstrumentPart
    {
        ModelPerformer[] performers;
        int keyFrom;
        int keyTo;
        int velFrom;
        int velTo;
        int exclusiveClass;
    }
}
